@@ -15,7 +15,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-__author__ = 'Nicolas Baer <nicolas.baer@uzh.ch>'
+__author__ = 'Nicolas Baer <nicolas.baer@uzh.ch>, Antonio Messina <antonio.s.messina@gmail.com>'
 
 # System imports
 import os
@@ -24,8 +24,14 @@ import sys
 
 # External modules
 from configobj import ConfigObj
-from voluptuous.voluptuous import message, MultipleInvalid, Invalid
-from voluptuous import Schema, All, Length, Any, Url, Boolean
+try:
+    # Voluptuous version >= 0.8.1
+    from voluptuous import message, MultipleInvalid, Invalid, Schema
+    from voluptuous import All, Length, Any, Url, Boolean
+except ImportError:
+    # Voluptuous version <= 0.7.2
+    from voluptuous.voluptuous import message, MultipleInvalid, Invalid
+    from voluptuous import Schema, All, Length, Any, Url, Boolean
 
 # Elasticluster imports
 from elasticluster.exceptions import ConfigurationError
@@ -111,6 +117,7 @@ class Configurator(object):
         provider = Configurator.cloud_providers_map[conf['provider']]
         providerconf = conf.copy()
         providerconf.pop('provider')
+        providerconf['storage_path'] = self.general_conf['storage']
 
         return provider(**providerconf)
 
@@ -297,7 +304,6 @@ class ConfigValidator(object):
         format if possible.
         """
         self._pre_validate()
-
         # custom validators
         @message("file could not be found")
         def check_file(v):
@@ -308,13 +314,7 @@ class ConfigValidator(object):
                 raise Invalid("file could not be found `%s`" % v)
 
         # schema to validate all cluster properties
-        schema = {"cloud": {"provider": 'ec2_boto',
-                            "ec2_url": Url(str),
-                            "ec2_access_key": All(str, Length(min=1)),
-                            "ec2_secret_key": All(str, Length(min=1)),
-                            "ec2_region": All(str, Length(min=1)),
-                            },
-                  "cluster": {"cloud": All(str, Length(min=1)),
+        schema = {"cluster": {"cloud": All(str, Length(min=1)),
                               "setup_provider": All(str, Length(min=1)),
                               "login": All(str, Length(min=1)),
                               },
@@ -330,6 +330,16 @@ class ConfigValidator(object):
                             }
                   }
 
+        cloud_schema_ec2 = {"provider": 'ec2_boto',
+                            "ec2_url": Url(str),
+                            "ec2_access_key": All(str, Length(min=1)),
+                            "ec2_secret_key": All(str, Length(min=1)),
+                            "ec2_region": All(str, Length(min=1))}
+        cloud_schema_gce = {"provider": 'google',
+                            "gce_client_id": All(str, Length(min=1)),
+                            "gce_client_secret": All(str, Length(min=1)),
+                            "gce_project_id": All(str, Length(min=1))}
+
         node_schema = {
             "flavor": All(str, Length(min=1)),
             "image_id": All(str, Length(min=1)),
@@ -339,6 +349,8 @@ class ConfigValidator(object):
         # validation
         validator = Schema(schema, required=True, extra=True)
         validator_node = Schema(node_schema, required=True, extra=True)
+        ec2_validator = Schema(cloud_schema_ec2, required=True, extra=False)
+        gce_validator = Schema(cloud_schema_gce, required=True, extra=False)
 
         if not self.config:
             raise Invalid("No clusters found in configuration.")
@@ -346,6 +358,11 @@ class ConfigValidator(object):
         for cluster, properties in self.config.iteritems():
             validator(properties)
 
+            if properties['cloud']['provider'] == "ec2":
+                ec2_validator(properties['cloud'])
+            elif properties['cloud']['provider'] == "google":
+                gce_validator(properties['cloud'])
+
             if 'nodes' not in properties or len(properties['nodes']) == 0:
                 raise Invalid(
                     "No nodes configured for cluster `%s`" % cluster)
@@ -381,6 +398,43 @@ class ConfigReader(object):
         self.configfile = configfile
         self.conf = ConfigObj(self.configfile, interpolation=False)
 
+        @message("file could not be found")
+        def check_file(v):
+            f = os.path.expanduser(os.path.expanduser(v))
+            if os.path.exists(f):
+                return f
+            else:
+                raise Invalid("file could not be found `%s`" % v)
+
+        self.schemas = {
+            "cloud": Schema(
+                {"provider": Any('ec2_boto', 'google'),
+                "ec2_url": Url(str),
+                "ec2_access_key": All(str, Length(min=1)),
+                "ec2_secret_key": All(str, Length(min=1)),
+                "ec2_region": All(str, Length(min=1)),
+                "gce_project_id": All(str, Length(min=1)),
+                "gce_client_id": All(str, Length(min=1)),
+                "gce_client_secret": All(str, Length(min=1)),
+                }),
+            "cluster": Schema(
+                {"cloud": All(str, Length(min=1)),
+                 "setup_provider": All(str, Length(min=1)),
+                 "login": All(str, Length(min=1)),
+                 }, required=True, extra=True),
+            "setup": Schema(
+                {"provider": All(str, Length(min=1)),
+                 }, required=True, extra=True),
+            "login": Schema(
+                {"image_user": All(str, Length(min=1)),
+                 "image_user_sudo": All(str, Length(min=1)),
+                 "image_sudo": Boolean(str),
+                 "user_key_name": All(str, Length(min=1)),
+                 "user_key_private": check_file(),
+                 "user_key_public": check_file(),
+                 }, required=True)
+            }
+
     def read_config(self):
         """
         Reads the configuration properties from the ini file and links the
@@ -396,27 +450,64 @@ class ConfigReader(object):
 
         conf_values = dict()
 
+        errors = MultipleInvalid()
+
         for cluster in clusters:
             name = re.search(ConfigReader.cluster_section + "/(.*)",
                              cluster).groups()[0]
+            if not name:
+                errors.add("Invalid section name `%s`" % cluster)
+                continue
+
+            cluster_conf = dict(self.conf[cluster])
+
             try:
-                cluster_conf = dict(self.conf[cluster])
-                cloud_name = ConfigReader.cloud_section + "/" + cluster_conf[
-                    'cloud']
-                login_name = ConfigReader.login_section + "/" + cluster_conf[
-                    'login']
-                setup_name = ConfigReader.setup_section + "/" + cluster_conf[
-                    'setup_provider']
+                self.schemas['cluster'](cluster_conf)
+            except MultipleInvalid as ex:
+                for error in ex.errors:
+                    errors.add("Section `%s`: %s" % (cluster, error))
+                continue
 
-                values = dict()
-                values['cluster'] = cluster_conf
+            cloud_name = ConfigReader.cloud_section + "/" + cluster_conf[
+                'cloud']
+            login_name = ConfigReader.login_section + "/" + cluster_conf[
+                'login']
+            setup_name = ConfigReader.setup_section + "/" + cluster_conf[
+                'setup_provider']
+
+            values = dict()
+            values['cluster'] = cluster_conf
+            try:
                 values['setup'] = dict(self.conf[setup_name])
-                values['login'] = dict(self.conf[login_name])
-                values['cloud'] = dict(self.conf[cloud_name])
+                self.schemas['setup'](values['setup'])
+            except KeyError, ex:
+                errors.add("cluster `%s` setup section `%s` does not exists" % (cluster, setup_name))
+            except MultipleInvalid, ex:
+                for error in ex.errors:
+                    errors.add(error)
 
+            try:
+                values['login'] = dict(self.conf[login_name])
+                self.schemas['login'](values['login'])
+            except KeyError, ex:
+                errors.add("cluster `%s` login section `%s` does not exists" % (cluster, login_name))
+            except MultipleInvalid, ex:
+                for error in ex.errors:
+                    errors.add(error)
+
+            try:
+                values['cloud'] = dict(self.conf[cloud_name])
+                self.schemas['cloud'](values['cloud'])
+            except KeyError, ex:
+                errors.add("cluster `%s` cloud section `%s` does not exists" % (cluster, cloud_name))
+            except MultipleInvalid, ex:
+                for error in ex.errors:
+                    errors.add(error)
+
+            try:
                 # nodes can inherit the properties of cluster or overwrite them
                 nodes = dict((key, value) for key, value in
-                             values['cluster'].iteritems() if
+                             values['cluster'].items() if
                              key.endswith('_nodes'))
                 values['nodes'] = dict()
                 for node in nodes.iterkeys():
@@ -434,9 +525,9 @@ class ConfigReader(object):
                         values['nodes'][node_name] = values['cluster']
 
                 conf_values[name] = values
-            except KeyError:
-                raise MultipleInvalid(
-                    "could not find all sections required `cluster`, "
-                    "`setup`, \ `login`, `cloud` for cluster `%s`" % name)
+            except KeyError, ex:
+                errors.add("Error in section `%s`" % cluster)
 
+        if errors.errors:
+            raise errors
         return conf_values
