@@ -23,7 +23,7 @@ import shutil
 import tempfile
 import unittest
 
-from voluptuous.voluptuous import MultipleInvalid
+from voluptuous import Invalid
 
 from elasticluster.conf import ConfigReader, ConfigValidator, Configurator
 from elasticluster.cluster import Node
@@ -227,6 +227,16 @@ class TestConfigValidator(unittest.TestCase):
     def tearDown(self):
         os.unlink(self.path)
 
+    def test_gce_config(self):
+        self.config['mycluster']['cloud'] = {
+            "provider": "google",
+            "gce_client_id": "***fill in your data here***",
+            "gce_client_secret": "***fill in your data here***",
+            "gce_project_id": "test-id"}
+
+        validator = ConfigValidator(self.config)
+        validator.validate()
+
 
     def test_valid_config(self):
         '''
@@ -243,13 +253,13 @@ class TestConfigValidator(unittest.TestCase):
         config = copy.deepcopy(self.config)
         config["mycluster"]["login"]["user_key_public"] = "/tmp/elastic-test"
         validator = ConfigValidator(config)
-        self.assertRaises(MultipleInvalid, validator.validate)
+        self.assertRaises(Invalid, validator.validate)
 
         # check wrong url
         config = copy.deepcopy(config)
         config["mycluster"]["setup"]["ec2_host"] = "www.elasticluster"
         validator = ConfigValidator(config)
-        self.assertRaises(MultipleInvalid, validator.validate)
+        self.assertRaises(Invalid, validator.validate)
 
         # check all mandatory properties
         optional = ["frontend_groups", "compute_groups", "frontend_nodes",
@@ -263,7 +273,7 @@ class TestConfigValidator(unittest.TestCase):
                         config_tmp = copy.deepcopy(config)
                         del config_tmp[cluster][section][property]
                         validator = ConfigValidator(config_tmp)
-                        self.assertRaises(MultipleInvalid, validator.validate)
+                        self.assertRaises(Invalid, validator.validate)
 
         # check all node properties
         mandatory = ["flavor", "image_id", "security_group"]
@@ -274,7 +284,7 @@ class TestConfigValidator(unittest.TestCase):
                     config_tmp = copy.deepcopy(config)
                     del config_tmp["mycluster"]["nodes"][node][property]
                     validator = ConfigValidator(config_tmp)
-                    self.assertRaises(MultipleInvalid, validator.validate)
+                    self.assertRaises(Invalid, validator.validate)
 
 
 class TestConfigReader(unittest.TestCase):
@@ -361,13 +371,13 @@ class TestConfigReader(unittest.TestCase):
             flavor=m1.small
             frontend_nodes=1
             compute_nodes=2
-            frontend_class=frontend
+            ssh_to=frontend
 
             [cluster/torque]
             cloud=hobbes
             frontend_nodes=1
             compute_nodes=2
-            frontend_class=frontend
+            ssh_to=frontend
             security_group=default
             # CentOS image
             image_id=ami-0000004f
@@ -452,9 +462,9 @@ class TestConfigReader(unittest.TestCase):
             flavor=m1.small
             frontend_nodes=1
             compute_nodes=2
-            frontend_class=frontend
+            ssh_to=frontend
             """
-        self.assertRaises(MultipleInvalid, self._check_read_config, config)
+        self.assertRaises(Invalid, self._check_read_config, config)
 
     def test_read_section_linking(self):
         '''
@@ -491,6 +501,6 @@ class TestConfigReader(unittest.TestCase):
             flavor=m1.small
             frontend_nodes=1
             compute_nodes=2
-            frontend_class=frontend
+            ssh_to=frontend
             """
-        self.assertRaises(MultipleInvalid, self._check_read_config, config)
+        self.assertRaises(Invalid, self._check_read_config, config)
